@@ -1,8 +1,10 @@
 /*
- * Copyright (C) 2014-2016 Fanout, Inc.
+ * Copyright (C) 2014-2017 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #ifndef STATSPACKET_H
@@ -58,6 +67,7 @@ public:
 	QByteArray itemId; // message
 	QByteArray transport; // message
 	int blocks; // message
+	int subscribers; // subscribed
 	int connectionsMax; // report
 	int connectionsMinutes; // report
 	int messagesReceived; // report
@@ -65,6 +75,7 @@ public:
 	int httpResponseMessagesSent; // report
 	int blocksReceived; // report
 	int blocksSent; // report
+	int duration; // report
 
 	StatsPacket() :
 		type((Type)-1),
@@ -73,13 +84,15 @@ public:
 		ssl(false),
 		ttl(-1),
 		blocks(-1),
+		subscribers(-1),
 		connectionsMax(-1),
 		connectionsMinutes(-1),
 		messagesReceived(-1),
 		messagesSent(-1),
 		httpResponseMessagesSent(-1),
 		blocksReceived(-1),
-		blocksSent(-1)
+		blocksSent(-1),
+		duration(-1)
 	{
 	}
 
