@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016-2017 Fanout, Inc.
+ * Copyright (C) 2016-2019 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
@@ -273,13 +273,13 @@ public:
 
 			if(!instruct.response.body.isEmpty())
 			{
-				// apply filters of all channels to content
+				// apply ProxyContent filters of all channels
 				QStringList allFilters;
 				foreach(const Instruct::Channel &c, instruct.channels)
 				{
 					foreach(const QString &filter, c.filters)
 					{
-						if(!allFilters.contains(filter))
+						if((Filter::targets(filter) & Filter::ProxyContent) && !allFilters.contains(filter))
 							allFilters += filter;
 					}
 				}
@@ -419,7 +419,7 @@ public:
 				QStringList contentFilters;
 				foreach(const QString &f, channels[item.channel].filters)
 				{
-					if(Filter::isContentFilter(f))
+					if(Filter::targets(f) & Filter::MessageContent)
 						contentFilters += f;
 				}
 				if(contentFilters != f.contentFilters)
@@ -519,16 +519,24 @@ private:
 		}
 	}
 
-	void setupKeepAliveTimer()
+	void setupKeepAlive()
 	{
 		if(instruct.keepAliveTimeout >= 0)
 		{
 			int timeout = instruct.keepAliveTimeout * 1000;
 			timeout = qMax(timeout - (qrand() % KEEPALIVE_RAND_MAX), 0);
+			timer->setSingleShot(true);
 			timer->start(timeout);
 		}
 	}
 
+	void adjustKeepAlive()
+	{
+		// if idle mode, restart the timer. else leave alone
+		if(timer && instruct.keepAliveMode == Instruct::Idle)
+			setupKeepAlive();
+	}
+
 	void prepareToClose()
 	{
 		state = Closing;
@@ -787,7 +795,7 @@ private:
 				QStringList contentFilters;
 				foreach(const QString &f, channels[item.channel].filters)
 				{
-					if(Filter::isContentFilter(f))
+					if(Filter::targets(f) & Filter::MessageContent)
 						contentFilters += f;
 				}
 				if(contentFilters != f.contentFilters)
@@ -830,7 +838,7 @@ private:
 				req->writeBody(body);
 
 				// restart keep alive timer
-				setupKeepAliveTimer();
+				adjustKeepAlive();
 
 				if(!nextUri.isEmpty() && instruct.nextLinkTimeout >= 0)
 				{
@@ -883,7 +891,7 @@ private:
 
 		// start keep alive timer, if it wasn't started already
 		if(!timer->isActive())
-			setupKeepAliveTimer();
+			setupKeepAlive();
 
 		if(!nextUri.isEmpty() && instruct.nextLinkTimeout >= 0)
 			updateManager->registerSession(q, instruct.nextLinkTimeout, nextUri);
@@ -1085,15 +1093,13 @@ private:
 
 				if(!c.prevId.isNull())
 				{
-					QString name = adata.channelPrefix + c.name;
-
 					if(!rp.haveInspectInfo)
 					{
 						rp.haveInspectInfo = true;
 						rp.inspectInfo.doProxy = true;
 					}
 
-					rp.inspectInfo.lastIds.insert(name.toUtf8(), c.prevId.toUtf8());
+					rp.inspectInfo.lastIds.insert(c.name.toUtf8(), c.prevId.toUtf8());
 				}
 			}
 
@@ -1409,13 +1415,13 @@ private slots:
 		{
 			haveOutReqHeaders = true;
 
-			// apply filters of all channels to content
+			// apply ProxyContent filters of all channels
 			QStringList allFilters;
 			foreach(const Instruct::Channel &c, instruct.channels)
 			{
 				foreach(const QString &filter, c.filters)
 				{
-					if(!allFilters.contains(filter))
+					if((Filter::targets(filter) & Filter::ProxyContent) && !allFilters.contains(filter))
 						allFilters += filter;
 				}
 			}
@@ -1475,6 +1481,8 @@ private slots:
 		{
 			req->writeBody(instruct.keepAliveData);
 
+			setupKeepAlive();
+
 			stats->addActivity(adata.route.toUtf8(), 1);
 		}
 	}
