@@ -1,8 +1,10 @@
 /*
- * Copyright (C) 2016 Fanout, Inc.
+ * Copyright (C) 2016-2017 Fanout, Inc.
  *
  * This file is part of Pushpin.
  *
+ * $FANOUT_BEGIN_LICENSE:AGPL$
+ *
  * Pushpin is free software: you can redistribute it and/or modify it under
  * the terms of the GNU Affero General Public License as published by the Free
  * Software Foundation, either version 3 of the License, or (at your option)
@@ -15,6 +17,13 @@
  *
  * You should have received a copy of the GNU Affero General Public License
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ *
+ * Alternatively, Pushpin may be used under the terms of a commercial license,
+ * where the commercial license agreement is provided with the software or
+ * contained in a written agreement between you and Fanout. For further
+ * information use the contact form at <https://fanout.io/enterprise/>.
+ *
+ * $FANOUT_END_LICENSE$
  */
 
 #ifndef SEQUENCER_H
@@ -33,8 +42,11 @@ public:
 	Sequencer(PublishLastIds *publishLastIds, QObject *parent = 0);
 	~Sequencer();
 
+	void setIdCacheTtl(int secs);
+
+	// seq = false means ID cache handling only
 	// note: may emit signals
-	void addItem(const PublishItem &item);
+	void addItem(const PublishItem &item, bool seq = true);
 
 	void clearPendingForChannel(const QString &channel);
 
